/* BFEncodingUtilities.m created by lindberg on Fri 28-Jan-2000 */

#import "BFEncodingUtilities.h"
#import <Foundation/NSString.h>
#import <Foundation/NSData.h>
#import <Foundation/NSException.h>
#import <string.h> // for strlen(), memset()
#import <stdio.h>  // for EOF
#import "FoundationAdditions.h"
#import "TGenAdditions.h" // for +stringWithCStringNoCopy:length:

/************************* Base 64 *************************/

/*
The tables and some of the code that used to be here was borrowed from
metamail, which comes with this message:

  Copyright (c) 1991 Bell Communications Research, Inc. (Bellcore)

  Permission to use, copy, modify, and distribute this material
  for any purpose and without fee is hereby granted, provided
  that the above copyright notice and this permission notice
  appear in all copies, and that the name of Bellcore not be
  used in advertising or publicity pertaining to this
  material without the specific, prior written permission
  of an authorized representative of Bellcore.	BELLCORE
  MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY
  OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS",
  WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
*/

static char basis_64[] =
   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

static inline void encode64chunk(const unsigned char *in, char *out, int limit)
{
    unsigned c1, c2, c3;

    c1 = in[0];
    c2 = (limit > 1)? in[1] : 0;
    c3 = (limit > 2)? in[2] : 0;

    *out++ = basis_64[c1 >> 2];
    *out++ = basis_64[((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4)];
    if (limit >= 3) {
        *out++ = basis_64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6)];
        *out++ = basis_64[c3 & 0x3F];
    } else if (limit == 2) {
        *out++ = basis_64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6)];
        *out++ = '=';
    } else {
        *out++ = '=';
        *out++ = '=';
    }
}


#define B64_LINE_LENGTH 72

char *BFBase64EncodeBytes(const char *bytes, int length, const char *newlineString, int *outlen)
{
    int			numChunks;
    int         outputSize;
    int			lineCount = 0;
    int         newlineLength = (newlineString == NULL)? 0 : strlen(newlineString);
    char		*outBuffer;
    const char	*inPointer;
    char		*outPointer;

    numChunks = (length+2) / 3;
    outputSize = numChunks * 4;

    if (newlineLength > 0) {
        int numNewlines = (outputSize+(B64_LINE_LENGTH)) / B64_LINE_LENGTH;
        outputSize += (numNewlines * newlineLength);
    }

    outBuffer  = NSZoneMalloc(NULL, outputSize);
    outPointer = outBuffer;	
    inPointer  = bytes;
    for (; length > 0; inPointer += 3, length -= 3)
    {
        encode64chunk((const unsigned char *)inPointer, outPointer, length);

        outPointer += 4;
        lineCount  += 4;

        if (newlineLength > 0 && lineCount >= B64_LINE_LENGTH)
        {
            memcpy(outPointer, newlineString, newlineLength);
            outPointer += newlineLength;
            lineCount = 0;
        }
    }

    /* Add last newline if needed */
    if (newlineLength > 0 && lineCount > 0)
        memcpy(outPointer, newlineString, newlineLength);

    if (outlen) *outlen = outputSize;
    return outBuffer;
}

NSString *BFBase64EncodeData(NSData *data, NSString *newline)
{
    int  outLength;
    char *bytes = BFBase64EncodeBytes([data bytes], [data length], [newline cString], &outLength);

    return [NSString stringWithCStringNoCopy:bytes length:outLength];
}
